#!/bin/bash
GREP='/usr/bin/grep'
MAINARG=$1
SECONDARG=$2
THIRDARG=$3
ORIGINALIPTABLES='~/.local/share/vpn.ac/iptables.original'

## Flush old rules.

function _test {
    echo 'TEST COMMAND EXECUTED'
}

function _enable {
    {

	if iptables -L -n |grep -q "vpnacallowedip"; then
	echo "iptables rules overwritten"
	else
        iptables-save > $ORIGINALIPTABLES
	fi
        iptables -F
        iptables -X
        iptables -t nat -F
        iptables -t nat -X
        iptables -t mangle -F
        iptables -t mangle -X
        iptables -N vpnacallowedip
        iptables -N lan

        ## Traffic on the loopback interface is accepted.
        ## Established incoming connections are accepted.
        iptables -A INPUT -m state --state RELATED,ESTABLISHED -j ACCEPT
        ## Allow connections on VPN and lo interfaces.
        iptables -A INPUT -i tun+ -j ACCEPT
        iptables -A OUTPUT -o tun+ -j ACCEPT
        iptables -A OUTPUT -o lo -j ACCEPT
        ## Existing connections are accepted.
        iptables -A OUTPUT -m state --state ESTABLISHED,RELATED -j ACCEPT
	## Allow traffic to VPN server inbound IP
        iptables -A vpnacallowedip -d $SECONDARG -j ACCEPT
        ## Reject all other outgoing traffic.
        iptables -A OUTPUT -j vpnacallowedip
        iptables -A OUTPUT -j lan
	iptables -A OUTPUT -j DROP
	## Allow DHCP in LAN.
        iptables -A OUTPUT -s 192.168.0.0/16,10.0.0.0/8,172.16.0.0/12 -p udp --sport 67:68 --dport 67:68 -j lan
    } &>/dev/null
    echo 'ENABLED'
}

function _restore {
    {
        iptables-restore < $ORIGINALIPTABLES
    } &>/dev/null
    echo 'RESTORED'
}

function _disable {
    {
        iptables -F
        iptables -X
    } &>/dev/null
    echo 'DISABLED'
}

function _replace {
    {
        iptables -F vpnacallowedip
        iptables -A vpnacallowedip -d $SECONDARG -j ACCEPT
        iptables -A vpnacallowedip -s $SECONDARG -j ACCEPT
    } &> /dev/null
    echo 'OK'
}

function _localnetwork_allow {
    #echo 'Allow LAN traffic'
    {
        iptables -A lan ! -o tun+ -s 192.168.0.0/16,10.0.0.0/8,172.16.0.0/12 -d 192.168.0.0/16,10.0.0.0/8,172.16.0.0/12 -j ACCEPT
    } &> /dev/null
    echo 'OK (LAN allowed)'
}

function _localnetwork_deny {
    #echo 'Deny LAN traffic'
    {
        iptables -F lan
    } &> /dev/null
    echo 'OK (LAN blocked)'
}

function _status {
    {
         RESULT=$(iptables -L -n)
    } &> /dev/null
    if [[ $RESULT =~ "vpnacallowedip" ]]; then
        echo 'ENABLED status'
    else
        echo 'DISABLED status'
    fi
}

case $MAINARG in
    enable)
        _enable
        ;;
    restore)
        _restore
        ;;
    disable)
        _disable
        ;;
    status)
        _status
        ;;
    replace)
        _replace
        ;;
    restore)
        _restore
        ;;
    test)
        _test
        ;;

    # Change localnetwork state
    localnetwork)
        case $SECONDARG in
            allow)
                _localnetwork_allow
                ;;
            deny)
                _localnetwork_deny
                ;;
            *)
        esac
        ;;
    *)
   echo 'Usage: vpn.ac-killswitch.sh {enable|restore|disable|replace [ server IP]|localnetwork allow|localnetwork deny}'
   exit 1
   ;;
esac
